@@ -14,12 +14,15 @@ class LED(utils.RGBObject):
     '''
 
     def __init__(self, data: utils.LEDData, color: utils.RGBColor, led_id: int, device_id: int, network_client: NetworkClient):
-        self.name = data.name
-        self.colors = [color]
-        self._colors = self.colors[:]
         self.id = led_id
         self.device_id = device_id
         self.comms = network_client
+        self._update(data, color)
+
+    def _update(self, data: utils.LEDData, color: utils.RGBColor):
+        self.name = data.name
+        self.colors = [color]
+        self._colors = self.colors[:]
 
     def set_color(self, color: utils.RGBColor, fast: bool = False):
         '''
@@ -45,17 +48,27 @@ class Zone(utils.RGBContainer):
     '''
 
     def __init__(self, data: utils.ZoneData, zone_id: int, device_id: int, network_client: NetworkClient):
+        self.leds = [None for led in data.leds]
+        self.device_id = device_id
+        self.comms = network_client
+        self.id = zone_id
+        self._update(data)
+
+    def _update(self, data: utils.ZoneData):
         self.name = data.name
         self.type = data.zone_type
-        self.leds = [LED(data.leds[x], data.colors[x], x + data.start_idx, device_id, network_client) for x in range(len(data.leds))]
+        if len(self.leds) != len(data.leds):
+            self.leds = [None for led in data.leds]
+        for x in range(len(data.leds)):
+            if self.leds[x] is None:
+                self.leds[x] = LED(data.leds[x], data.colors[x], x, self.device_id, self.comms)
+            else:
+                self.leds[x]._update(data.leds[x], data.colors[x])
         self.mat_width = data.mat_width
         self.mat_height = data.mat_height
         self.matrix_map = data.matrix_map
         self.colors = data.colors
         self._colors = self.colors[:]
-        self.device_id = device_id
-        self.comms = network_client
-        self.id = zone_id
 
     def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0, fast: bool = False):
         '''
@@ -124,19 +137,34 @@ class Device(utils.RGBContainer):
     '''
 
     def __init__(self, data: utils.ControllerData, device_id: int, network_client: NetworkClient):
+        self.leds = [None for i in data.leds]
+        self.zones = [None for i in data.zones]
+        self.id = device_id
+        self.device_id = device_id
+        self.comms = network_client
+        self._update(data)
+
+    def _update(self, data: utils.ControllerData):
         self.name = data.name
         self.metadata = data.metadata
         self.type = data.device_type
-        self.leds = [LED(data.leds[x], data.colors[x], x, device_id, network_client) for x in range(len(data.leds))]
-        self.zones = [Zone(data.zones[x], x, device_id, network_client) for x in range(len(data.zones))]
+        if len(self.leds) != len(data.leds):
+            self.leds = [None for i in data.leds]
+        for x in range(len(data.leds)):
+            if self.leds[x] is None:
+                self.leds[x] = LED(data.leds[x], data.colors[x], x, self.device_id, self.comms)
+            else:
+                self.leds[x]._update(data.leds[x], data.colors[x])
+        for x in range(len(data.zones)):
+            if self.zones[x] is None:
+                self.zones[x] = Zone(data.zones[x], x, self.device_id, self.comms)
+            else:
+                self.zones[x]._update(data.zones[x])
         self.modes = data.modes
         self.colors = data.colors
         self._colors = self.colors[:]
         self.active_mode = data.active_mode
         self.data = data
-        self.id = device_id
-        self.device_id = device_id
-        self.comms = network_client
 
     def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0, fast: bool = False):
         '''
@@ -195,7 +223,10 @@ class Device(utils.RGBContainer):
         elif type(mode) == int:
             mode = self.modes[mode]
         elif type(mode) == str:
-            mode = next((m for m in self.modes if m.name.lower() == mode.lower()))
+            try:
+                mode = next((m for m in self.modes if m.name.lower() == mode.lower()))
+            except StopIteration as e:
+                raise ValueError(f"Mode `{mode}` not found for device `{self.name}`") from e
         data = mode.pack(self.comms._protocol_version)
         self.comms.send_header(
             self.id,
@@ -228,18 +259,21 @@ class OpenRGBClient(utils.RGBObject):
         '''
         self.device_num = 0
         self.devices = []
+        self.profiles = []
         self.comms = NetworkClient(self._callback, address, port, name, protocol_version)
         self.address = address
         self.port = port
         self.name = name
         self.comms.requestDeviceNum()
         while any((dev is None for dev in self.devices)):
-            sleep(.2)
+            sleep(.1)
+        if self.comms._protocol_version >= 2:
+            self.update_profiles()
 
     def __repr__(self):
         return f"OpenRGBClient(address={self.address}, port={self.port}, name={self.name})"
 
-    def _callback(self, device: int, type: int, data: Optional[Union[int, utils.ControllerData]]):
+    def _callback(self, device: int, type: int, data: Optional):
         if type == utils.PacketType.REQUEST_CONTROLLER_COUNT:
             if data != self.device_num or data != len(self.devices):
                 self.device_num = data
@@ -251,12 +285,14 @@ class OpenRGBClient(utils.RGBObject):
                 if self.devices[device] is None:
                     self.devices[device] = Device(data, device, self.comms)
                 else:
-                    self.devices[device].__init__(data, device, self.comms)
+                    self.devices[device]._update(data)
             except IndexError:
                 self.comms.requestDeviceNum()
         elif type == utils.PacketType.DEVICE_LIST_UPDATED:
             self.device_num = 0
             self.comms.requestDeviceNum()
+        elif type == utils.PacketType.REQUEST_PROFILE_LIST:
+            self.profiles = data
 
     def set_color(self, color: utils.RGBColor, fast: bool = False):
         '''
@@ -276,47 +312,116 @@ class OpenRGBClient(utils.RGBObject):
         '''
         return [device for device in self.devices if device.type == type]
 
-    def load_profile(self, name: str, directory: str = ''):
+    def get_devices_by_name(self, name: str, exact: bool = True) -> list[Device]:
         '''
-        Loads an OpenRGB profile file
+        Gets a list of any devices matching the requested name
 
-        :param name: the name of the profile
+        :param name: the name of the device(s) you want to get
+        :param exact: whether to check for only a precise match or accpet a device that contains name
+        '''
+        if exact:
+            return [device for device in self.devices if device.name == name]
+        return [device for device in self.devices if name.lower() in device.name.lower()]
+
+    def load_profile(self, name: Union[str, int, utils.Profile], local: bool = False, directory: str = ''):
+        '''
+        Loads an OpenRGB profile
+
+        :param name: Can be a profile's name, index, or even the Profile itself
+        :param local: Whether to load a local file or a profile from the server.
         :param directory: what directory the profile is in.  Defaults to HOME/.config/OpenRGB
         '''
-        if directory == '':
-            directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
-        with open(f'{directory}/{name}.orp', 'rb') as f:
-            controllers = utils.Profile.unpack(f).controllers
-            pairs = []
-            for device in self.devices:
-                for new_controller in controllers:
-                    if new_controller.name == device.name \
-                            and new_controller.device_type == device.type \
-                            and new_controller.metadata.description == device.metadata.description:
-                        controllers.remove(new_controller)
-                        pairs.append((new_controller, device))
-            # print("Pairs:")
-            for new_controller, device in pairs:
-                # print(device.name, new_controller.name)
-                if new_controller.colors != device.colors:
-                    device.set_colors(new_controller.colors)
-                # print(new_controller.active_mode)
-                if new_controller.active_mode != device.active_mode:
-                    device.set_mode(new_controller.active_mode)
+        if local:
+            assert type(name) is str
+            if directory == '':
+                directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
+            with open(f'{directory}/{name}.orp', 'rb') as f:
+                controllers = utils.LocalProfile.unpack(f).controllers
+                pairs = []
+                for device in self.devices:
+                    for new_controller in controllers:
+                        if new_controller.name == device.name \
+                                and new_controller.device_type == device.type \
+                                and new_controller.metadata.description == device.metadata.description:
+                            controllers.remove(new_controller)
+                            pairs.append((new_controller, device))
+                # print("Pairs:")
+                for new_controller, device in pairs:
+                    # print(device.name, new_controller.name)
+                    if new_controller.colors != device.colors:
+                        device.set_colors(new_controller.colors)
+                    # print(new_controller.active_mode)
+                    if new_controller.active_mode != device.active_mode:
+                        device.set_mode(new_controller.active_mode)
+        else:
+            if self.comms._protocol_version < 2:
+                raise utils.SDKVersionError("Your version of OpenRGB doesn't support SDK profile controls")
+            if type(name) is str:
+                try:
+                    name = next(p for p in self.profiles if p.name.lower() == name.lower())
+                except StopIteration as e:
+                    raise ValueError(f"`{name}` is not an existing profile") from e
+            elif type(name) is int:
+                name = self.profiles[name]
+            elif type(name) is utils.Profile:
+                pass
+            name = name.pack()
+            self.comms.send_header(0, utils.PacketType.REQUEST_LOAD_PROFILE, len(name))
+            self.comms.send_data(name)
 
-    def save_profile(self, name: str, directory: str = ''):
+    def save_profile(self, name: Union[str, int, utils.Profile], local: bool = False, directory: str = ''):
         '''
-        Saves the current state of all of your devices to an OpenRGB profile
-        file
+        Saves the current state of all of your devices to a new or existing
+        OpenRGB profile
 
-        :param name: the name of the profile to save
+        :param name: Can be a profile's name, index, or even the Profile itself
+        :param local: Whether to load a local file or a profile on the server.
         :param directory: what directory to save the profile in.  Defaults to HOME/.config/OpenRGB
         '''
-        self.update()
-        if directory == '':
-            directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
-        with open(f'{directory.rstrip("/")}/{name}.orp', 'wb') as f:
-            f.write(utils.Profile([dev.data for dev in self.devices]).pack())
+        if local:
+            self.update()
+            if directory == '':
+                directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
+            with open(f'{directory.rstrip("/")}/{name}.orp', 'wb') as f:
+                f.write(utils.Profile([dev.data for dev in self.devices]).pack())
+        else:
+            if self.comms._protocol_version < 2:
+                raise utils.SDKVersionError("Your version of OpenRGB doesn't support SDK profile controls")
+            if type(name) is str:
+                try:
+                    name = next(p for p in self.profiles if p.name.lower() == name.lower())
+                except StopIteration:
+                    name = utils.Profile(name)
+            elif type(name) is int:
+                name = self.profiles[name]
+            elif type(name) is utils.Profile:
+                pass
+            name = name.pack()
+            self.comms.send_header(0, utils.PacketType.REQUEST_SAVE_PROFILE, len(name))
+            self.comms.send_data(name)
+            self.update_profiles()
+
+    def delete_profile(self, name: Union[str, int, utils.Profile]):
+        '''
+        Deletes the selected profile
+
+        :param name: Can be a profile's name, index, or even the Profile itself
+        '''
+        if self.comms._protocol_version < 2:
+            raise utils.SDKVersionError("Your version of OpenRGB doesn't support SDK profile controls")
+        if type(name) is str:
+            try:
+                name = next(p for p in self.profiles if p.name.lower() == name.lower())
+            except StopIteration as e:
+                raise ValueError(f"`{name}` is not an existing profile") from e
+        elif type(name) is int:
+            name = self.profiles[name]
+        elif type(name) is utils.Profile:
+            pass
+        name = name.pack()
+        self.comms.send_header(0, utils.PacketType.REQUEST_DELETE_PROFILE, len(name))
+        self.comms.send_data(name)
+        self.update_profiles()
 
     def update(self):
         '''
@@ -328,6 +433,14 @@ class OpenRGBClient(utils.RGBObject):
         for x in range(self.device_num):
             self.comms.requestDeviceData(x)
 
+    def update_profiles(self):
+        '''
+        Gets the list of available profiles from the server.
+        '''
+        if self.comms._protocol_version < 2:
+            raise utils.SDKVersionError("Your version of OpenRGB doesn't support SDK profile controls")
+        self.comms.requestProfileList()
+
     def show(self, fast: bool = False, force: bool = False):
         '''
         Shows all devices
